
// This code controls 4 simultaneous steppers
// It reads data from the serial monitor, that is sent through Bluetooth, 
// and controls the steppers accordingly
//  THis code runs all steppers at the same time.
//
// Copyright (C) 2018 Wael Khalil

#include <TimerOne.h>

//include the library code
#include <AccelStepper.h> //stepper motors library

//include the library code
#include <Servo.h> //servo motor library

#include <Wire.h>
#include <FaBo9Axis_MPU9250.h>// MPU 9250 library

#include <SPI.h>
#include <SD.h>// MicroSD library

//Define final name
File myFile;

FaBo9Axis fabo_9axis;

//Define pins used for the Servo Motor
const int ServoPin = 7;

Servo myServo; // create servo object to control a servo

//define some contant numbers that will be used in the code
const int MaxSpeedSlow = 300;
const int AccelerationSlow = 100;
const int SlowDistance = 10000;

bool stopped = true;

const int Initialize_Distance = 50;

// Define some steppers and the pins they will use
AccelStepper stepper1(1, 8, 9); 
AccelStepper stepper2(1, 5, 6); 
AccelStepper stepper3(1, 3, 2); 
AccelStepper stepper4(1, A2, 4); 

void doSteps()
{
  if(stopped == false)
  {
    stepper1.runSpeed();
    stepper2.runSpeed();
    stepper3.runSpeed();
    stepper4.runSpeed();
  }
    // Toggle LED
    digitalWrite( 13, digitalRead( 13 ) ^ 1 );
}

void setup()
{  
  pinMode(13,OUTPUT);
  // initialize serial:
  Serial.begin(9600);

  //Define servo pin
  myServo.attach(7);

  //initialize servo position
  myServo.write(90);
  
  delay (2000);


  //set initial stepper settings
  stepper1.setMaxSpeed(MaxSpeedSlow);
  stepper1.setAcceleration(AccelerationSlow);
  
  stepper2.setMaxSpeed(MaxSpeedSlow);
  stepper2.setAcceleration(AccelerationSlow);
  
  stepper3.setMaxSpeed(MaxSpeedSlow);
  stepper3.setAcceleration(AccelerationSlow);
  
  stepper4.setMaxSpeed(MaxSpeedSlow);
  stepper4.setAcceleration(AccelerationSlow);



 Timer1.initialize(250); // set a timer of length 100000 microseconds (or 0.1 sec - or 10Hz => the led will blink 5 times, 5 cycles of on-and-off, per second)
  Timer1.attachInterrupt( doSteps ); // attach the service routine here
  
  //Initialize Robot
  stepper1.moveTo(-Initialize_Distance);
  stepper2.moveTo(Initialize_Distance);
  stepper3.moveTo(Initialize_Distance);
  stepper4.moveTo(-Initialize_Distance);
 
  stepper1.runToPosition();
  stepper2.runToPosition();
  stepper3.runToPosition();
  stepper4.runToPosition();
 
  delay(5000);
if (fabo_9axis.begin()) {
    Serial.println("configured FaBo 9Axis I2C Brick");
  } else {
    Serial.println("device error");
    while(1);
  }

  // open the file. note that only one file can be open at a time,
  // so you have to close this one before opening another.
  myFile = SD.open("PIPE_Data.txt", FILE_WRITE);

  // if the file opened okay, write to it:
  if (myFile) {
    Serial.print("Writing to PIPE_Data.txt...");
    myFile.println("Initializing 1, 2, 3.");
    
    // close the file:
    myFile.close();
    Serial.println("done.");

}
}

  


void MPU() {
  float ax,ay,az;
  float gx,gy,gz;
  float mx,my,mz;
  float temp;

  fabo_9axis.readAccelXYZ(&ax,&ay,&az);
  fabo_9axis.readGyroXYZ(&gx,&gy,&gz);
  fabo_9axis.readMagnetXYZ(&mx,&my,&mz);
  fabo_9axis.readTemperature(&temp);

  myFile.print("ax: ");
  myFile.print(ax);
  myFile.print(" ay: ");
  myFile.print(ay);
  myFile.print(" az: ");
  myFile.println(az);

  myFile.print("gx: ");
  myFile.print(gx);
  myFile.print(" gy: ");
  myFile.print(gy);
  myFile.print(" gz: ");
  myFile.println(gz);

  myFile.print("mx: ");
  myFile.print(mx);
  myFile.print(" my: ");
  myFile.print(my);
  myFile.print(" mz: ");
  myFile.println(mz);

  myFile.print("temp: ");
  myFile.println(temp);

  delay(1000);
}

void forward() 
{
  //set stepper settings
  stepper1.setSpeed(-MaxSpeedSlow);
  
  stepper2.setSpeed(MaxSpeedSlow);
  
  stepper3.setSpeed(MaxSpeedSlow);
  
  stepper4.setSpeed(-MaxSpeedSlow);
}

void back() 
{
  //set stepper settings
  stepper1.setSpeed(MaxSpeedSlow);
  
  stepper2.setSpeed(-MaxSpeedSlow);
  
  stepper3.setSpeed(-MaxSpeedSlow);
  
  stepper4.setSpeed(MaxSpeedSlow);
}


void left() 
{
  //set stepper settings
  stepper1.setSpeed(-MaxSpeedSlow);
  
  stepper2.setSpeed(-MaxSpeedSlow);
  
  stepper3.setSpeed(MaxSpeedSlow);
  
  stepper4.setSpeed(MaxSpeedSlow);  
}



void right() 
{
  //set stepper settings
  stepper1.setSpeed(MaxSpeedSlow);
  
  stepper2.setSpeed(MaxSpeedSlow);
  
  stepper3.setSpeed(-MaxSpeedSlow);
  
  stepper4.setSpeed(-MaxSpeedSlow);  
}



void forwardleft() 
{
  //set stepper settings
  stepper1.setSpeed(-MaxSpeedSlow);
  
  stepper2.setSpeed(0);
  
  stepper3.setSpeed(MaxSpeedSlow);
  
  stepper4.setSpeed(0);  
}



void forwardright() 
{
  //set stepper settings
  stepper1.setSpeed(0);
  
  stepper2.setSpeed(MaxSpeedSlow);
  
  stepper3.setSpeed(0);
  
  stepper4.setSpeed(-MaxSpeedSlow);  
}



void backright() 
{
  //set stepper settings
  stepper1.setSpeed(MaxSpeedSlow);
  
  stepper2.setSpeed(0);
  
  stepper3.setSpeed(-MaxSpeedSlow);
  
  stepper4.setSpeed(0);  
}



void backleft() 
{
 //set stepper settings
  stepper1.setSpeed(0);
  
  stepper2.setSpeed(-MaxSpeedSlow);
  
  stepper3.setSpeed(0);
  
  stepper4.setSpeed(MaxSpeedSlow);
}



void rotateLeft() 
{
 //set stepper settings
  stepper1.setSpeed(MaxSpeedSlow);
  
  stepper2.setSpeed(MaxSpeedSlow);
  
  stepper3.setSpeed(MaxSpeedSlow);
  
  stepper4.setSpeed(MaxSpeedSlow);  
}

void rotateRight() 
{
 //set stepper settings
  stepper1.setSpeed(-MaxSpeedSlow);
  
  stepper2.setSpeed(-MaxSpeedSlow);
  
  stepper3.setSpeed(-MaxSpeedSlow);
  
  stepper4.setSpeed(-MaxSpeedSlow); 
}

void Stop() 
{
  stepper1.stop();
  stepper2.stop();
  stepper3.stop();
  stepper4.stop();
}



void loop()
{
  MPU();
 while (Serial.available())
  {
    char inChar = (char)Serial.read();
     
  
    switch(inChar) 
    {
      case 'F':
      stopped = false;
        forward();
        break;
       
      case 'G':
      stopped = false;
        back();
        break;
       
      case 'L':
      stopped = false;
        left();
        break;
       
      case 'R':
      stopped = false;
        right();
        break;
      
      case 'Q':
      stopped = false;
        forwardleft();
        break;
      
      case 'E':
      stopped = false;
        forwardright();
        break;
     
      case 'Z':
      stopped = false;
        backleft();
        break;
      
      case 'C':
      stopped = false;
        backright();
        break;

        case 'X':
      stopped = false;
        rotateLeft();
        break;
      
      case 'Y':
      stopped = false;
        rotateRight();
        break;
              
       case 'S':
       stopped=true;
       Stop();
       break; 
    }
  }
}





